﻿package 
{
	public class Enemy extends GameObject
	{
		private var color:uint;
		
		public var velocity:Vector2;
		
		public var enemyType:int;
		
		protected var removable:Boolean;
		
		protected var fsm:StateMachine;
		
		public function Enemy(filename:String)
		{
			super();
			
			enemyType = EnemyType.DEFAULT;
			
			// Set the radius and color
			this.radius = GameConstants.ENEMY_RADIUS;
			this.color = 0x00cc00;
			this.removable = false;
			
			fsm = new StateMachine();
			
			// Initialize the enemy
			init(filename);
		}
		
		private function init(filename:String):void
		{
			//load enemy art
			this.load(GameConstants.OBJECT_DIR + filename );
			
			fsm = new StateMachine();
		}
		
		public override function update():void
		{
			
		}
		
		public function walk()
		{
			fsm.changeState(new WalkState(this));
		}
		
		public function dodge()
		{
			fsm.changeState(new DodgeState(this));
		}
		
		public function pause()
		{
			fsm.changeState(new PauseState(this));
		}
		
		public function returnToSidewalk() 
		{
			fsm.changeState(new ReturnState(this));
		}
		
		public function Panic(dir:Vector2)
		{
			fsm.changeState(new PanicState(this, dir));
		}
		
		public function avoid()
		{
			fsm.changeState(new AvoidState(this));
		}
		
		protected function drawBoundingCircle():void
		{			
			this.graphics.clear();
			
			this.graphics.beginFill(0xcc0000, 0.3);
			this.graphics.drawCircle(0, 0, radius);
			this.graphics.endFill();
		}
		
		public function getColor():uint
		{
			return color;
		}
		
		public function getStateMachine():StateMachine
		{
			return fsm;
		}
		
		public static function getValidPos(testPos:Vector2):Vector2
		{
			if (testPos.y > GameConstants.SIDEWALK_BOTTOM - GameConstants.ENEMY_RADIUS)
			{
				return new Vector2(testPos.x, GameConstants.SIDEWALK_BOTTOM - GameConstants.ENEMY_RADIUS);
			}
			else
			{
				return testPos;
			}
		}
		
		public function speedUp()
		{
			// Increase the speed
			velocity.multiply(GameConstants.HUMAN_SPEED_MULTIPLIER);
		}
		
		public function resetSpeed()
		{
			this.velocity.setLength(GameConstants.ENEMY_WALKING_SPEED);
		}
		
		public function hit() 
		{
			if (fsm.getCurrentState() is PauseState || fsm.getCurrentState() is DodgeState)
			{
				fsm.changeState(new DodgeState(this));
			}
			else
			{
				// Randomly react to being hit
				if (Math.random() > GameConstants.ENEMY_DODGE_ODDS)
				{
					speedUp();
				}
				else
				{
					fsm.changeState(new DodgeState(this));
				}
			}
		}
		
		public function kill()
		{
			fsm.changeState(new DeadState(this));
		}
		
		public function isDodging()
		{
			return (fsm.getCurrentState() is DodgeState);
		}
		
		public function isPanicking()
		{
			return (fsm.getCurrentState() is PanicState);
		}
		
		public function isDead()
		{
			return (fsm.getCurrentState() is DeadState);
		}
		
		public function isRemovable()
		{
			return this.removable;
		}
		
		public function markForRemoval()
		{
			this.removable = true;
		}
	}	
}