﻿package 
{
	public class EnemyManager
	{
		private static var instance:EnemyManager;
		
		private var enemies:Array;
		
		private var nextHumanSpawn:int;
		private var nextCarSpawn:int;
		private var enemyCnt:int;
		private var carCnt:int;
		
		public function EnemyManager() 
		{
			// Create an array to store enemies
			enemies = new Array();
			
			// Create a random spawn counter
			nextHumanSpawn = (int)(Math.random() * GameConstants.MAX_HUMAN_SPAWN_TIME);
			nextCarSpawn = (int)(Math.random() * GameConstants.MAX_CAR_SPAWN_TIME);
		}
		
		public static function getInstance()
		{
			if (instance == null)
			{
				instance = new EnemyManager();
			}
			
			return instance;
		}
		
		public function update():void
		{
			// Decrement the spawn counter
			nextHumanSpawn--;
			nextCarSpawn--;
			
			updateThreats();
			
			// Update all enemies
			updateEnemies();
			
			// If it is time to spawn a new enemy
			if (nextHumanSpawn <= 0)
			{
				// Spawn a new enemy
				spawnHuman();
			}
			if (nextCarSpawn <= 0)
			{
				// Spawn a new enemy
				spawnCar();
			}
		}
		
		private function updateEnemies() 
		{
			// If the enemy list is not empty
			if (enemies.length > 0)
			{
				// Update each enemy
				for (var i:int = 0; i < enemies.length; i++)
				{
					enemies[i].update();
				}
				
				//remove dead enemies and/or enemies of the stage
				this.removeDeadEnemies();
			}
		}
		
		//spawn new human enemy
		private function spawnHuman()
		{
			//if not too many enemies on stage
			//decide enemy type
			var type:int = Math.random() * EnemyType.HUMAN;
				
			// Create a new enemy
			var enemy:Enemy = EnemyFactory.getInstance().createEnemy(type);
			
			// Add the enemy to the main stage
			Game.mainStage.addChildAt(enemy, GameConstants.LAYER);
				
			// While this enemy collides with any other enemy
			if (collidesWithEnemy(enemy))
			{
				trace("Colliding... respawning...");
				Game.mainStage.removeChild(enemy);
				enemy = EnemyFactory.getInstance().createEnemy(enemy.enemyType);
			}
			
			enemy.walk();
			
			// Add the enemy to the enemy array
			enemies.push(enemy);
			
			// Create a new random spawn time
			nextHumanSpawn = (int)(Math.random() * GameConstants.MAX_HUMAN_SPAWN_TIME);
			
			enemyCnt++;
		}
		
		private function spawnCar()
		{
			//if not too many enemies on stage
			//decide enemy type
			var type:int;
			
			var car:Car;
			
			var rand:int = Math.round(Math.random() * 2 - 1);
			
			switch (rand)
			{
				case -1:
					type = (int)(Math.random() * 3 + 1);
					car = CarFactory.getInstance().createCarLeft(type);		
					
					// Add the enemy to the main stage
					Game.mainStage.addChildAt(car, GameConstants.LAYER);
					
					car.walk();
					
					// Add the enemy to the enemy array
					enemies.push(car);
					break;
				case 0:
					type = (int)(Math.random() * 3 + 1);
					car = CarFactory.getInstance().createCarLeft(type);
					
					// Add the enemy to the main stage
					Game.mainStage.addChildAt(car, GameConstants.LAYER);
					
					car.walk();
					
					// Add the enemy to the enemy array
					enemies.push(car);
					
					type = (int)(Math.random() * 3 + 1);
					car = CarFactory.getInstance().createCarRight(type);
					
					// Add the enemy to the main stage
					Game.mainStage.addChildAt(car, GameConstants.LAYER);
					
					car.walk();
					
					// Add the enemy to the enemy array
					enemies.push(car);
					break;
				case 1:
					type = (int)(Math.random() * 3 + 1);
					car = CarFactory.getInstance().createCarRight(type);
					// Add the enemy to the main stage
					Game.mainStage.addChildAt(car, GameConstants.LAYER);
					
					car.walk();
					
					// Add the enemy to the enemy array
					enemies.push(car);
					break;
			}
			
			var spawnMultipler:Number = GameConstants.MAX_CAR_SPAWN_TIME - GameConstants.MIN_CAR_SPAWN_TIME
			
			// Create a new random spawn time
			nextCarSpawn = (int)(Math.random() * spawnMultipler + GameConstants.MIN_CAR_SPAWN_TIME);
			
			enemyCnt++;
		}
		
		public function updateThreats():void
		{
			var threatened:Boolean = false;
			var dir:Vector2 = Vector2.Zero;
			var temp:Vector2;
			
			for (var i:int = 0; i < enemies.length; i++)
			{
				if (!(enemies[i] is Human) || enemies[i].isPanicking() || enemies[i].isDead())
				{
					continue;
				}
				
				if (Math.random() > enemies[i].exciteProb)
				{
					continue;
				}
				
				var rad:Number = enemies[i].getRadius() + GameConstants.PANIC_RADIUS_BUF;
				
				for (var j:int = 0; j < enemies.length; j++)
				{					
					if (i == j)
					{
						continue;
					}
					
					temp = Vector2.subtract(enemies[i].getPos(), enemies[j].getPos());
					
					if (temp.length() <= rad)
					{
						if (enemies[j].isDodging() || enemies[j].isPanicking())
						{
							threatened = true;
							
							dir.add(temp);
						}
					}
					
					if (threatened)
					{
						enemies[i].Panic(dir);
						
						dir.normalize();
					}
				}
				
				threatened = false;
			}
		}
		
		public function collide(obj:GameObject) 
		{
			// For each enemy
			for (var i:int = 0; i < enemies.length; i++)
			{
				if (obj == enemies[i])
				{
					continue;
				}
				
				// If the item hit the enemy
				if (collides(obj, enemies[i]))
				{
					if (obj is Item)
					{
						if (!enemies[i].isDead())
						{
							// Make the enemy respond to being hit
							enemies[i].hit();
						}
					}
					else if (enemies[i] is Human && obj is Car)
					{
						if (!enemies[i].isDead() && !(obj as Car).isDead())
						{
							// Kill the enemy
							enemies[i].kill();
							var plusOne:PlusOne = new PlusOne();
							plusOne.setPos(enemies[i].getPos());
							Game.mainStage.addChild(plusOne);
							ItemManager.getInstance().addItem(plusOne);
						}
					}
					else if (enemies[i] is Car && obj is Car)
					{
						enemies[i].hit();
					}
				}
			}
		}
		
		private function collides(obj:GameObject, enemy:Enemy):Boolean
		{
			return enemy.hitTestObject(obj);
		}
		
		private function collidesWithEnemy(obj:GameObject):Boolean
		{
			// For each enemy
			for (var i:int = 0; i < enemies.length; i++)
			{
				if (obj is Enemy)
				{
					if ((obj as Enemy) == enemies[i])
					{
						continue;
					}
				}
				
				if ((obj.getPos().x < 0 || obj.getPos().y > GameConstants.STAGE_SIZE.x) ||
					(enemies[i].getPos().x < 0 || enemies[i].getPos().y > GameConstants.STAGE_SIZE.x))
				{
					return false;
				}
				
				// If the item hit the enemy
				if (collides(obj, enemies[i]))
				{
					// Return true
					return true;
				}
			}
			
			// If there were no collisions, return false
			return false;
		}
		
		//sort array by pushing all invalid enemies to back of array
		private function removeDeadEnemies()
		{
			for (var i:int = 0; i < enemies.length; )
			{
				if (enemies[i].isRemovable())
				{
					Game.mainStage.removeChild(enemies[i]);
					enemies.splice(i, 1);
				}
				else i++;
			}
		}
		
		public function clear()
		{
			for (var i:int = 0; i < enemies.length; i++)
			{
				Game.mainStage.removeChild(enemies[i]);
			}
			enemies = new Array();
		}
	}	
}