﻿package 
{
	import flash.display.MovieClip;
	import flash.text.TextField;
	import flash.utils.Timer;
	
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.ui.Mouse;
	import flash.events.MouseEvent;
	
	public class Game extends MovieClip
	{
		private var enemyManager:EnemyManager;
		private var itemManager:ItemManager;
		private var soundManager:SoundManager;
		
		private static var isOver:Boolean;
		
		public static var mainStage:MovieClip
		public static var player:Player;
		public static var uiTimerKillCnt:UI;
		public static var target:GameObject;
		public static var killCount:int;
		public static var killCountText:TextField;
		public static var timer:Timer;
		
		public function Game(stage:MovieClip)
		{
			// Get the main stage
			mainStage = new MovieClip;
			
			this.addChild(mainStage);
			
			// Create a new enemy manager
			enemyManager = EnemyManager.getInstance();
			
			// Create a new item manager
			itemManager = ItemManager.getInstance();
			
			// Create a new sound manager
			soundManager = SoundManager.getInstance();
			
			// Initialize the main manager
			init();
		}
		
		public function init():void
		{			
			killCount = 0;
			
			isOver = false;
			
			var background:Background = new Background();
			mainStage.addChildAt(background, 0);
			
			// Create a new player
			player = new Player();
			player.setPos( new Vector2( 313, 409 ) );
			player.y = GameConstants.STAGE_SIZE.y /*- player.getRadius()*/ - 60;
			mainStage.addChild(player);
			
			//create shadows
			var shadows:GameObject = new GameObject();
			shadows.load(GameConstants.OBJECT_DIR + "Shadows.swf");
			shadows.setPos( new Vector2( 0, 0 ) );
			shadows.alpha = 0.6;
			mainStage.addChild(shadows);
			
			//create tree
			var tree:GameObject = new GameObject();
			tree.load(GameConstants.OBJECT_DIR + "Tree.swf");
			tree.setPos( new Vector2( 154, 218 ) );
			mainStage.addChild(tree);
			
			//create awning
			var awning:GameObject = new GameObject();
			awning.load(GameConstants.OBJECT_DIR + "Awning.swf");
			awning.setPos( new Vector2( 268, 280 ) );
			mainStage.addChild(awning);
			
			//create target
			target = new GameObject();
			target.load(GameConstants.OBJECT_DIR + "Target.swf");
			target.setPos( new Vector2( mouseX, mouseY ) );
			mainStage.addChild(target);
			
			mainStage.addEventListener(MouseEvent.MOUSE_MOVE, OnMouseMove);
			mainStage.addEventListener(MouseEvent.CLICK, ItemManager.onMouseClick);
		}
		
		public function startGame()
		{
			//create timer
			uiTimerKillCnt = new UI();
			mainStage.addChild(uiTimerKillCnt);
			
			//create killCountText
			killCountText = new TextField();
			killCountText.x = 100;
			killCountText.y = 430;
			killCountText.scaleX = killCountText.scaleY = 3;
			killCountText.textColor = 0xFFFFFF;
			killCountText.text = "0000";
			mainStage.addChild(killCountText);
		}
		
		public function update():void
		{
			if (uiTimerKillCnt != null)
			{
				uiTimerKillCnt.update();
				
				if (uiTimerKillCnt.isTimeExpired())
				{
					isOver = true;
				}
			}
			
			if (MainManager.gameState == GameState.GAME_PLAY)
			{
				var str:String = killCount.toString();
				if ( killCount < 1000) str = "0" + str;
				if ( killCount < 100) str = "0" + str;
				if ( killCount < 10) str = "0" + str;
				
				killCountText.text = str;
			}	
			
			// Update the enemy manager
			enemyManager.update();
				
			// Update the item manager
			itemManager.update();
			
		}
		
		private function OnMouseMove(event:MouseEvent):void
		{
			//set mouse
			target.x = mouseX;
			target.y = mouseY;
			
			// Set the player x-position to the mouse x-position
			player.x = mouseX;
			
			// Reposition the player onto the roof if it moves off
			if (player.x + (player.width/2) > GameConstants.STAGE_SIZE.x - GameConstants.BUILDING_EDGE)
			{
				player.x = GameConstants.STAGE_SIZE.x - GameConstants.BUILDING_EDGE - (player.width/2);
			}
			
			if (player.x - (player.width/2) < GameConstants.BUILDING_EDGE)
			{
				player.x = (player.width/2) + GameConstants.BUILDING_EDGE;
			}
		}
		
		public static function incrementScore()
		{
			if (!isOver)
			{
				killCount++;
			}
		}
		
		public function reset()
		{
			killCount = 0;
			
			isOver = false;
			
			enemyManager.clear();
		}
		
		public function isGameOver():Boolean
		{
			return isOver;
		}
	}
}