﻿package 
{
	import flash.display.MovieClip;
	import flash.events.MouseEvent;
	
	public class ItemManager
	{
		private static var instance:ItemManager;
		
		private static var items:Array;
		
		public function ItemManager() 
		{
			// Create an array to store items
			items = new Array();
		}
		
		public static function getInstance():ItemManager
		{
			if (instance == null)
			{
				instance = new ItemManager();
			}
			
			return instance;
		}
		
		public function update()
		{
			// Update all items
			updateItems();
		}
		
		private function updateItems():void
		{
			// If the items list is not empty
			if (items.length > 0)
			{
				// Update each enemy
				for (var i:int = 0; i < items.length; i++)
				{
					items[i].update();
				}
				
				// Create a new array of dead items
				var toRemove:Array = items.filter(isItemDead);
				
				// Remove all dead items from the items list
				items = items.filter(isItemAlive);
				
				for (var j:int = 0; j < toRemove.length; j++)
				{					
					// If this item is a splash
					if (toRemove[j] is Balloon)
					{
						SoundManager.getInstance().playSound(SoundMappings.BALLOON_SPLATTER);
						
						// Remove the item from the main stage
						Game.mainStage.removeChild(toRemove[j]);
						
						// See if the item collided with an enemy
						EnemyManager.getInstance().collide(toRemove[j]);
						
						// Create a new splash
						var splatter:BalloonSplatter = new BalloonSplatter();
						splatter.setPos(toRemove[j].getPos());
						
						// Add the splash to the splash list
						items.push(splatter);
						
						// Add the splash to the main stage
						Game.mainStage.addChild(splatter);
					}
					else
					{
						// Remove the item from the main stage
						Game.mainStage.removeChild(toRemove[j]);
					}
				}
			}
		}
		
		private function isItemAlive(elem:*, i:int, a:Array):Boolean
		{
			return (elem.lifespan > 0);
		}
		
		private function isItemDead(elem:*, i:int, a:Array):Boolean
		{
			return (elem.lifespan <= 0);
		}
		
		public static function onMouseClick(event:MouseEvent):void
		{
			if (MainManager.gameState != GameState.GAME_PLAY)
			{
				return;
			}
			
			// If the mouse click was inside the building, bail out
			if (event.stageY >= GameConstants.SIDEWALK_BOTTOM)
			{
				return;
			}
			
			// Create a new item
			var balloon:Balloon = (ItemFactory.getInstance().createItem(ItemType.BALLOON) as Balloon);
			
			// Initialize the item
			balloon.init(GameConstants.FALL_FRAMES, new Vector2(Game.player.x + GameConstants.BALLOON_OFFSET, Game.player.y - GameConstants.BALLOON_OFFSET), new Vector2(event.stageX, event.stageY));
			
			// Push the item onto the item array
			items.push(balloon);
			
			// Add the item to the main stage
			Game.mainStage.addChild(balloon);
			
			//play throw balloon animation
			Game.player.playAnimation("throw");
		}
		
		public function addItem(item:Item):void
		{
			items.push(item);
		}
	}
}