﻿package 
{
	public class Vector2
	{
		public var x:Number;
		public var y:Number;
		
		public static const Zero:Vector2 = new Vector2(0, 0);
		
		public function Vector2(x:Number, y:Number) 
		{
			// Store the x and y
			this.x = x;
			this.y = y;
		}
		
		public function length():Number
		{
			// Return the length of the vector
			return (Math.sqrt(x * x + y * y));
		}
		
		public function lengthSquared():Number
		{
			// Return the square of the length of the vector
			return (x * x + y * y);
		}
		
		public function setLength(newLength:Number):void
		{
			this.normalize();
			this.multiply(newLength);
		}
		
		public function normalize():void
		{
			var length = this.length();
			
			x /= length;
			y /= length;
		}
		
		public function subtract(v1:Vector2):void
		{
			// subtract the given vector from this vector
			this.x -= v1.x;
			this.y -= v1.y;
		}
		
		public function add(v1:Vector2):void
		{
			// Add the given vector to this vector
			this.x += v1.x;
			this.y += v1.y;
		}
		
		public function multiply(s:Number):void
		{
			// Multiply this vector by a given scalar
			this.x *= s;
			this.y *= s;
		}
		
		public static function subtract(v1:Vector2, v2:Vector2):Vector2
		{
			// Return a new vector containing the difference of two vectors
			return new Vector2(v2.x - v1.x, v2.y - v1.y);
		}
		
		public static function add(v1:Vector2, v2:Vector2):Vector2
		{
			// Return a new vector containing the sum of two vectors
			return new Vector2(v1.x + v2.x, v1.y + v2.y);
		}
		
		public static function multiply(s:Number, v:Vector2):Vector2
		{
			// Return a new vector containing the product of a vector and a scalar
			return new Vector2(s * v.x, s * v.y);
		}
		
		public function dot(v2:Vector2):Number
		{
			return (x * v2.x + y * v2.y);
		}
		
		public static function dot(v1:Vector2, v2:Vector2):Number
		{
			return (v1.x * v2.x + v1.y * v2.y);
		}
		
		public function getUnitVector():Vector2
		{
			var length:Number = this.length();
			
			return new Vector2(x / length, y / length);
		}
		
		public function clone():Vector2
		{
			// Return a copy of this vector
			return new Vector2(x, y);
		}
	}	
}