﻿package 
{
	public class Vector3
	{
		public var x:Number;
		public var y:Number;
		public var z:Number;
		
		public static const Zero:Vector3 = new Vector3(0, 0, 0);
		
		public function Vector3(x:Number, y:Number, z:Number)
		{
			this.x = x;
			this.y = y;
			this.z = z;
		}
		
		public function length():Number
		{
			// Return the length of the vector
			return (Math.sqrt(x * x + y * y + z * z));
		}
		
		public function lengthSquared():Number
		{
			// Return the square of the length of the vector
			return (x * x + y * y + z * z);
		}
		
		public function setLength(newLength:Number):void
		{
			this.normalize();
			this.multiply(newLength);
		}
		
		public function normalize():void
		{
			var length = this.length();
			
			x /= length;
			y /= length;
			z /= length;
		}
		
		public function subtract(v1:Vector3):void
		{
			// subtract the given vector from this vector
			this.x -= v1.x;
			this.y -= v1.y;
			this.z -= v1.z;
		}
		
		public function add(v1:Vector3):void
		{
			// Add the given vector to this vector
			this.x += v1.x;
			this.y += v1.y;
			this.z += v1.z;
		}
		
		public function multiply(s:Number):void
		{
			// Multiply this vector by a given scalar
			this.x *= s;
			this.y *= s;
			this.z *= s;
		}
		
		public static function subtract(v1:Vector3, v2:Vector3):Vector3
		{
			// Return a new vector containing the difference of two vectors
			return new Vector3(v2.x - v1.x, v2.y - v1.y, v2.z - v1.z);
		}
		
		public static function add(v1:Vector3, v2:Vector3):Vector3
		{
			// Return a new vector containing the sum of two vectors
			return new Vector3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
		}
		
		public static function multiply(s:Number, v:Vector3):Vector3
		{
			// Return a new vector containing the product of a vector and a scalar
			return new Vector3(s * v.x, s * v.y, s * v.z);
		}
		
		public function dot(v2:Vector3):Number
		{
			return (x * v2.x + y * v2.y + z * v2.z);
		}
		
		public static function dot(v1:Vector3, v2:Vector3):Number
		{
			return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
		}
		
		public function getCrossProduct(v2:Vector3):Vector3
		{
			return new Vector3(this.y * v2.z - this.z * v2.y, this.z * v2.x - this.x * v2.z, this.x * v2.y - this.y * v2.x);
		}
		
		public static function cross(v1:Vector3, v2:Vector3):Vector3
		{
			return new Vector3(v1.y * v2.z - v1.z * v2.y, v1.z * v2.x - v1.x * v2.z, v1.x * v2.y - v1.y * v2.x);
		}
		
		public function getUnitVector():Vector3
		{
			var length:Number = this.length();
			
			return new Vector3(x / length, y / length, z / length);
		}
		
		public function clone():Vector3
		{
			// Return a copy of this vector
			return new Vector3(x, y, z);
		}
	}
}